@@ -1,8 +1,9 @@
 from __future__ import print_function, division, absolute_import
 
 import io
-import os
+from glob import has_magic
 import logging
+import os
 import re
 from .compression import compr
 from .utils import (
@@ -84,6 +85,7 @@ class OpenFile(object):
                 self.compression,
                 self.encoding,
                 self.errors,
+                self.newline,
             ),
         )
 
@@ -147,6 +149,59 @@ class OpenFile(object):
         _close(self.fobjects, self.mode)
 
 
+class OpenFiles(list):
+    """List of OpenFile instances
+
+    Can be used in a single context, which opens and closes all of the
+    contained files. Normal list access to get the elements works as
+    normal.
+
+    A special case is made for caching filesystems - the files will
+    be down/uploaded together at the start or end of the context, and
+    this may happen concurrently, if the target filesystem supports it.
+    """
+
+    def __init__(self, *args, mode="rb", fs=None):
+        self.mode = mode
+        self.fs = fs
+        self.files = []
+        super().__init__(*args)
+
+    def __enter__(self):
+        if self.fs is None:
+            raise ValueError("Context has already been used")
+
+        fs = self.fs
+        while True:
+            if hasattr(fs, "open_many"):
+                # check for concurrent cache download; or set up for upload
+                self.files = fs.open_many(self)
+                return self.files
+            if hasattr(fs, "fs") and fs.fs is not None:
+                fs = fs.fs
+            else:
+                break
+        return [s.__enter__() for s in self]
+
+    def __exit__(self, *args):
+        fs = self.fs
+        if "r" not in self.mode:
+            while True:
+                if hasattr(fs, "open_many"):
+                    # check for concurrent cache upload
+                    fs.commit_many(self.files)
+                    self.files.clear()
+                    return
+                if hasattr(fs, "fs") and fs.fs is not None:
+                    fs = fs.fs
+                else:
+                    break
+        [s.__exit__(*args) for s in self]
+
+    def __repr__(self):
+        return "<List of %s OpenFile instances>" % len(self)
+
+
 def _close(fobjects, mode):
     for f in reversed(fobjects):
         if "r" not in mode and not f.closed:
@@ -166,6 +221,7 @@ def open_files(
     protocol=None,
     newline=None,
     auto_mkdir=True,
+    expand=True,
     **kwargs
 ):
     """ Given a path or paths, return a list of ``OpenFile`` objects.
@@ -204,6 +260,7 @@ def open_files(
     auto_mkdir: bool (True)
         If in write mode, this will ensure the target directory exists before
         writing, by calling ``fs.mkdirs(exist_ok=True)``.
+    expand: bool
     **kwargs: dict
         Extra options that make sense to a particular storage connection, e.g.
         host, port, username, password, etc.
@@ -217,28 +274,9 @@ def open_files(
 
     Returns
     -------
-    List of ``OpenFile`` objects.
+    An ``OpenFiles`` instance, which is a ist of ``OpenFile`` objects that can
+    be used as a single context
     """
-    chain = _un_chain(urlpath, kwargs)
-    if len(chain) > 1:
-        kwargs = chain[0][2]
-        inkwargs = kwargs
-        urlpath = False
-        for i, ch in enumerate(chain):
-            urls, protocol, kw = ch
-            if isinstance(urls, str):
-                if not urlpath and split_protocol(urls)[1]:
-                    urlpath = protocol + "://" + split_protocol(urls)[1]
-            else:
-                if not urlpath and any(split_protocol(u)[1] for u in urls):
-                    urlpath = [protocol + "://" + split_protocol(u)[1] for u in urls]
-            if i == 0:
-                continue
-            inkwargs["target_protocol"] = protocol
-            inkwargs["target_options"] = kw.copy()
-            inkwargs["fo"] = urls
-            inkwargs = inkwargs["target_options"]
-        protocol = chain[0][1]
     fs, fs_token, paths = get_fs_token_paths(
         urlpath,
         mode,
@@ -246,22 +284,27 @@ def open_files(
         name_function=name_function,
         storage_options=kwargs,
         protocol=protocol,
+        expand=expand,
     )
     if "r" not in mode and auto_mkdir:
         parents = {fs._parent(path) for path in paths}
         [fs.makedirs(parent, exist_ok=True) for parent in parents]
-    return [
-        OpenFile(
-            fs,
-            path,
-            mode=mode,
-            compression=compression,
-            encoding=encoding,
-            errors=errors,
-            newline=newline,
-        )
-        for path in paths
-    ]
+    return OpenFiles(
+        [
+            OpenFile(
+                fs,
+                path,
+                mode=mode,
+                compression=compression,
+                encoding=encoding,
+                errors=errors,
+                newline=newline,
+            )
+            for path in paths
+        ],
+        mode=mode,
+        fs=fs,
+    )
 
 
 def _un_chain(path, kwargs):
@@ -284,15 +327,31 @@ def _un_chain(path, kwargs):
         if "::" in path
         else [path]
     )
+    if len(bits) < 2:
+        return []
     # [[url, protocol, kwargs], ...]
-    return [
-        (
-            bit,
-            split_protocol(bit)[0] or "file",
-            kwargs.get(split_protocol(bit)[0] or "file", {}),
-        )
-        for bit in bits
-    ]
+    out = []
+    previous_bit = None
+    previous_protocol = None
+    for bit in reversed(bits):
+        protocol = split_protocol(bit)[0] or "file"
+        cls = get_filesystem_class(protocol)
+        extra_kwargs = cls._get_kwargs_from_urls(bit)
+        kws = kwargs.get(split_protocol(bit)[0] or "file", {})
+        kw = dict(**extra_kwargs, **kws)
+        if (
+            protocol in {"blockcache", "filecache", "simplecache"}
+            and "target_protocol" not in kw
+        ):
+            bit = previous_bit.replace(previous_protocol, protocol, 1)
+        out.append((bit, protocol, kw))
+        previous_bit = bit
+        previous_protocol = protocol
+    out = list(reversed(out))
+    # We should only do the url rewrite if the cache is in the middle of the chain
+    if out[0][1] in {"blockcache", "filecache", "simplecache"}:
+        out[0] = (f"{out[0][1]}://", out[0][1], out[0][2])
+    return out
 
 
 def url_to_fs(url, **kwargs):
@@ -375,6 +434,7 @@ def open(
         errors,
         protocol,
         newline=newline,
+        expand=False,
         **kwargs
     )[0]
 
@@ -396,8 +456,14 @@ def open_local(url, mode="rb", **storage_options):
     if "r" not in mode:
         raise ValueError("Can only ensure local files when reading")
     of = open_files(url, mode=mode, **storage_options)
-    paths = [f.open().name for f in of]
-    if isinstance(url, str) and "*" not in url:
+    if not getattr(of[0].fs, "local_file", False):
+        raise ValueError(
+            "open_local can only be used on a filesystem which"
+            " has attribute local_file=True"
+        )
+    with of as files:
+        paths = [f.name for f in files]
+    if isinstance(url, str) and not has_magic(url):
         return paths[0]
     return paths
 
@@ -466,7 +532,13 @@ def expand_paths_if_needed(paths, mode, num, fs, name_function):
 
 
 def get_fs_token_paths(
-    urlpath, mode="rb", num=1, name_function=None, storage_options=None, protocol=None
+    urlpath,
+    mode="rb",
+    num=1,
+    name_function=None,
+    storage_options=None,
+    protocol=None,
+    expand=True,
 ):
     """Filesystem, deterministic token, and paths from a urlpath and options.
 
@@ -487,7 +559,33 @@ def get_fs_token_paths(
         Additional keywords to pass to the filesystem class.
     protocol: str or None
         To override the protocol specifier in the URL
+    expand: bool
+        Expand string paths for writing, assuming the path is a directory
     """
+    if isinstance(urlpath, (list, tuple, set)):
+        urlpath = [stringify_path(u) for u in urlpath]
+    else:
+        urlpath = stringify_path(urlpath)
+    chain = _un_chain(urlpath, storage_options or {})
+    if len(chain) > 1:
+        storage_options = chain[0][2]
+        inkwargs = storage_options
+        urlpath = False
+        for i, ch in enumerate(chain):
+            urls, protocol, kw = ch
+            if isinstance(urls, str):
+                if not urlpath and split_protocol(urls)[1]:
+                    urlpath = protocol + "://" + split_protocol(urls)[1]
+            else:
+                if not urlpath and any(split_protocol(u)[1] for u in urls):
+                    urlpath = [protocol + "://" + split_protocol(u)[1] for u in urls]
+            if i == 0:
+                continue
+            inkwargs["target_protocol"] = protocol
+            inkwargs["target_options"] = kw.copy()
+            inkwargs["fo"] = urls
+            inkwargs = inkwargs["target_options"]
+        protocol = chain[0][1]
     if isinstance(urlpath, (list, tuple)):
         if not urlpath:
             raise ValueError("empty urlpath sequence")
@@ -522,7 +620,7 @@ def get_fs_token_paths(
         update_storage_options(options, storage_options)
         fs = cls(**options)
 
-        if "w" in mode:
+        if "w" in mode and expand:
             paths = _expand_paths(path, name_function, num)
         elif "*" in path:
             paths = [f for f in sorted(fs.glob(path)) if not fs.isdir(f)]
