@@ -6,6 +6,7 @@ import hashlib
 from shutil import move, rmtree
 import tempfile
 import inspect
+
 from fsspec import AbstractFileSystem, filesystem
 from fsspec.spec import AbstractBufferedFile
 from fsspec.core import MMapCache, BaseCache
@@ -113,6 +114,12 @@ class CachingFileSystem(AbstractFileSystem):
         self.load_cache()
         self.fs = fs if fs is not None else filesystem(target_protocol, **self.kwargs)
 
+        def _strip_protocol(path):
+            # acts as a method, since each instance has a difference target
+            return self.fs._strip_protocol(type(self)._strip_protocol(path))
+
+        self._strip_protocol = _strip_protocol
+
     def _mkcache(self):
         os.makedirs(self.storage[-1], exist_ok=True)
 
@@ -201,7 +208,7 @@ class CachingFileSystem(AbstractFileSystem):
             fn = os.path.join(storage, detail["fn"])
             if os.path.exists(fn):
                 return detail, fn
-        return False, None
+        return False
 
     def clear_cache(self):
         """Remove all files and metadat from the cache
@@ -275,9 +282,10 @@ class CachingFileSystem(AbstractFileSystem):
                 cache_options=cache_options,
                 **kwargs
             )
-        detail, fn = self._check_file(store_path)
+        detail = self._check_file(store_path)
         if detail:
             # file is in cache
+            detail, fn = detail
             hash, blocks = detail["fn"], detail["blocks"]
             if blocks is True:
                 # stored file is complete
@@ -286,7 +294,7 @@ class CachingFileSystem(AbstractFileSystem):
             # TODO: action where partial file exists in read-only cache
             logger.debug("Opening partially cached copy of %s" % path)
         else:
-            hash = hash_name(path, self.same_names)
+            hash = self.hash_name(path, self.same_names)
             fn = os.path.join(self.storage[-1], hash)
             blocks = set()
             detail = {
@@ -331,6 +339,9 @@ class CachingFileSystem(AbstractFileSystem):
         self.save_cache()
         return f
 
+    def hash_name(self, path, same_name):
+        return hash_name(path, same_name=same_name)
+
     def close_and_update(self, f, close):
         """Called when a file is closing, so store the set of blocks"""
         path = self._strip_protocol(f.path)
@@ -365,15 +376,16 @@ class CachingFileSystem(AbstractFileSystem):
             "pop_from_cache",
             "_mkcache",
             "local_file",
+            "_paths_from_path",
+            "open_many",
+            "commit_many",
+            "hash_name",
         ]:
             # all the methods defined in this class. Note `open` here, since
             # it calls `_open`, but is actually in superclass
             return lambda *args, **kw: getattr(type(self), item)(self, *args, **kw)
         if item in ["__reduce_ex__"]:
             raise AttributeError
-        if item in ["_strip_protocol"]:
-            # class methods
-            return lambda *args, **kw: getattr(type(self), item)(*args, **kw)
         if item in ["_cache"]:
             # class attributes
             return getattr(type(self), item)
@@ -418,7 +430,58 @@ class WholeFileCacheFileSystem(CachingFileSystem):
     protocol = "filecache"
     local_file = True
 
-    def _open(self, path, mode="rb", **kwargs):
+    def open_many(self, open_files):
+        paths, store_paths = zip(*[self._paths_from_path(of.path) for of in open_files])
+        if "r" in open_files.mode:
+            self._mkcache()
+        else:
+            return [
+                LocalTempFile(self.fs, path, mode=open_files.mode, autocommit=False)
+                for path in paths
+            ]
+
+        if self.compression:
+            raise NotImplementedError
+        details = [self._check_file(sp) for sp in store_paths]
+        downpath = [p for p, d in zip(paths, details) if not d]
+        downstore = [p for p, d in zip(store_paths, details) if not d]
+        downfn0 = [
+            os.path.join(self.storage[-1], self.hash_name(p, self.same_names))
+            for p, d in zip(paths, details)
+        ]  # keep these path names for opening later
+        downfn = [fn for fn, d in zip(downfn0, details) if not d]
+        if downpath:
+            # skip if all files are already cached and up to date
+            self.fs.get(downpath, downfn)
+
+            # update metadata - only happens when downloads are successful
+            newdetail = [
+                {
+                    "fn": self.hash_name(path, self.same_names),
+                    "blocks": True,
+                    "time": time.time(),
+                    "uid": self.fs.ukey(path),
+                }
+                for path in downpath
+            ]
+            self.cached_files[-1].update(
+                {store_path: detail for store_path, detail in zip(downstore, newdetail)}
+            )
+            self.save_cache()
+
+        def firstpart(fn):
+            # helper to adapt both whole-file and simple-cache
+            return fn[1] if isinstance(fn, tuple) else fn
+
+        return [
+            open(firstpart(fn0) if fn0 else fn1, mode=open_files.mode)
+            for fn0, fn1 in zip(details, downfn0)
+        ]
+
+    def commit_many(self, open_files):
+        self.fs.put([f.fn for f in open_files], [f.path for f in open_files])
+
+    def _paths_from_path(self, path):
         path = self._strip_protocol(path)
 
         if not path.startswith(self.target_protocol):
@@ -426,10 +489,15 @@ class WholeFileCacheFileSystem(CachingFileSystem):
         else:
             store_path = path
         path = self.fs._strip_protocol(store_path)
+        return path, store_path
+
+    def _open(self, path, mode="rb", **kwargs):
+        path, store_path = self._paths_from_path(path)
         if "r" not in mode:
             return self.fs._open(path, mode=mode, **kwargs)
-        detail, fn = self._check_file(store_path)
+        detail = self._check_file(store_path)
         if detail:
+            detail, fn = detail
             hash, blocks = detail["fn"], detail["blocks"]
             if blocks is True:
                 logger.debug("Opening local copy of %s" % path)
@@ -440,12 +508,11 @@ class WholeFileCacheFileSystem(CachingFileSystem):
                     "as a wholly cached file" % path
                 )
         else:
-            hash = hash_name(path, self.same_names)
+            hash = self.hash_name(path, self.same_names)
             fn = os.path.join(self.storage[-1], hash)
-            blocks = True
             detail = {
                 "fn": hash,
-                "blocks": blocks,
+                "blocks": True,
                 "time": time.time(),
                 "uid": self.fs.ukey(path),
             }
@@ -471,8 +538,9 @@ class WholeFileCacheFileSystem(CachingFileSystem):
             if getattr(f, "blocksize", 0) and f.size:
                 # opportunity to parallelise here
                 data = True
+                block = getattr(f, "blocksize", 5 * 2 ** 20)
                 while data:
-                    data = f.read(f.blocksize)
+                    data = f.read(block)
                     f2.write(data)
             else:
                 # this only applies to HTTP, should instead use streaming
@@ -481,7 +549,7 @@ class WholeFileCacheFileSystem(CachingFileSystem):
         return self._open(path, mode)
 
 
-class SimpleCacheFileSystem(CachingFileSystem):
+class SimpleCacheFileSystem(WholeFileCacheFileSystem):
     """Caches whole remote files on first access
 
     This class is intended as a layer over any other file system, and
@@ -512,7 +580,7 @@ class SimpleCacheFileSystem(CachingFileSystem):
 
     def _check_file(self, path):
         self._check_cache()
-        sha = hash_name(path, self.same_names)
+        sha = self.hash_name(path, self.same_names)
         for storage in self.storage:
             fn = os.path.join(storage, sha)
             if os.path.exists(fn):
@@ -538,39 +606,30 @@ class SimpleCacheFileSystem(CachingFileSystem):
         if fn:
             return open(fn, mode)
 
-        sha = hash_name(path, self.same_names)
+        sha = self.hash_name(path, self.same_names)
         fn = os.path.join(self.storage[-1], sha)
         logger.debug("Copying %s to local cache" % path)
         kwargs["mode"] = mode
 
         self._mkcache()
-        with self.fs._open(path, **kwargs) as f, open(fn, "wb") as f2:
-            if isinstance(f, AbstractBufferedFile):
-                # want no type of caching if just downloading whole thing
-                f.cache = BaseCache(0, f.cache.fetcher, f.size)
-            if getattr(f, "blocksize", 0) and f.size:
-                # opportunity to parallelise here (if not compressed)
-                if self.compression:
-                    comp = (
-                        infer_compression(path)
-                        if self.compression == "infer"
-                        else self.compression
-                    )
-                    f = compr[comp](f, mode="rb")
+        if self.compression:
+            with self.fs._open(path, **kwargs) as f, open(fn, "wb") as f2:
+                if isinstance(f, AbstractBufferedFile):
+                    # want no type of caching if just downloading whole thing
+                    f.cache = BaseCache(0, f.cache.fetcher, f.size)
+                comp = (
+                    infer_compression(path)
+                    if self.compression == "infer"
+                    else self.compression
+                )
+                f = compr[comp](f, mode="rb")
                 data = True
                 while data:
-                    data = f.read(f.blocksize)
+                    block = getattr(f, "blocksize", 5 * 2 ** 20)
+                    data = f.read(block)
                     f2.write(data)
-            else:
-                # this only applies to HTTP, should instead use streaming
-                if self.compression:
-                    comp = (
-                        infer_compression(path)
-                        if self.compression == "infer"
-                        else self.compression
-                    )
-                    f = compr[comp](f, mode="rb")
-                f2.write(f.read())
+        else:
+            self.fs.get(path, fn)
         return self._open(path, mode)
 
 
