@@ -2,6 +2,7 @@ from __future__ import print_function, division, absolute_import
 
 import aiohttp
 import asyncio
+import logging
 import re
 import requests
 import weakref
@@ -14,10 +15,11 @@ from ..caching import AllBytes
 # https://stackoverflow.com/a/15926317/3821154
 ex = re.compile(r"""<a\s+(?:[^>]*?\s+)?href=(["'])(.*?)\1""")
 ex2 = re.compile(r"""(http[s]?://[-a-zA-Z0-9@:%_+.~#?&/=]+)""")
+logger = logging.getLogger("fsspec.http")
 
 
-async def get_client():
-    return aiohttp.ClientSession()
+async def get_client(**kwargs):
+    return aiohttp.ClientSession(**kwargs)
 
 
 class HTTPFileSystem(AsyncFileSystem):
@@ -42,6 +44,7 @@ class HTTPFileSystem(AsyncFileSystem):
         cache_options=None,
         asynchronous=False,
         loop=None,
+        client_kwargs=None,
         **storage_options
     ):
         """
@@ -59,9 +62,12 @@ class HTTPFileSystem(AsyncFileSystem):
             When doing ls/glob, if this is True, only consider paths that have
             http/https matching the input URLs.
         size_policy: this argument is deprecated
+        client_kwargs: dict
+            Passed to aiohttp.ClientSession, see
+            https://docs.aiohttp.org/en/stable/client_reference.html
+            For example, ``{'auth': aiohttp.BasicAuth('user', 'pass')}``
         storage_options: key-value
-            May be credentials, e.g., `{'auth': ('username', 'pword')}` or any
-            other parameters passed on to requests
+            Any other parameters passed on to requests
         cache_type, cache_options: defaults used in open
         """
         super().__init__(self, asynchronous=asynchronous, loop=loop, **storage_options)
@@ -70,9 +76,10 @@ class HTTPFileSystem(AsyncFileSystem):
         self.same_schema = same_scheme
         self.cache_type = cache_type
         self.cache_options = cache_options
+        self.client_kwargs = client_kwargs or {}
         self.kwargs = storage_options
         if not asynchronous:
-            self._session = sync(self.loop, get_client)
+            self._session = sync(self.loop, get_client, **self.client_kwargs)
             weakref.finalize(self, sync, self.loop, self.session.close)
         else:
             self._session = None
@@ -84,7 +91,7 @@ class HTTPFileSystem(AsyncFileSystem):
         return self._session
 
     async def set_session(self):
-        self._session = await get_client()
+        self._session = await get_client(**self.client_kwargs)
 
     @classmethod
     def _strip_protocol(cls, path):
@@ -96,6 +103,7 @@ class HTTPFileSystem(AsyncFileSystem):
         # ignoring URL-encoded arguments
         kw = self.kwargs.copy()
         kw.update(kwargs)
+        logger.debug(url)
         async with self.session.get(url, **self.kwargs) as r:
             r.raise_for_status()
             text = await r.text()
@@ -140,7 +148,10 @@ class HTTPFileSystem(AsyncFileSystem):
     async def _cat_file(self, url, **kwargs):
         kw = self.kwargs.copy()
         kw.update(kwargs)
+        logger.debug(url)
         async with self.session.get(url, **kw) as r:
+            if r.status == 404:
+                raise FileNotFoundError(url)
             r.raise_for_status()
             out = await r.read()
         return out
@@ -148,7 +159,10 @@ class HTTPFileSystem(AsyncFileSystem):
     async def _get_file(self, rpath, lpath, chunk_size=5 * 2 ** 20, **kwargs):
         kw = self.kwargs.copy()
         kw.update(kwargs)
+        logger.debug(rpath)
         async with self.session.get(rpath, **self.kwargs) as r:
+            if r.status == 404:
+                raise FileNotFoundError(rpath)
             r.raise_for_status()
             with open(lpath, "wb") as fd:
                 chunk = True
@@ -160,6 +174,7 @@ class HTTPFileSystem(AsyncFileSystem):
         kw = self.kwargs.copy()
         kw.update(kwargs)
         try:
+            logger.debug(path)
             r = await self.session.get(path, **kw)
             async with r:
                 return r.status < 400
@@ -248,6 +263,10 @@ class HTTPFileSystem(AsyncFileSystem):
                 raise FileNotFoundError(url)
         return {"name": url, "size": size or None, "type": "file"}
 
+    def isdir(self, path):
+        # override, since all URLs are (also) files
+        return bool(self.ls(path))
+
 
 class HTTPFile(AbstractBufferedFile):
     """
@@ -357,6 +376,7 @@ class HTTPFile(AbstractBufferedFile):
         kwargs = self.kwargs.copy()
         headers = kwargs.pop("headers", {}).copy()
         headers["Range"] = "bytes=%i-%i" % (start, end - 1)
+        logger.debug(self.url + " : " + headers["Range"])
         r = await self.session.get(self.url, headers=headers, **kwargs)
         async with r:
             if r.status == 416:
