@@ -2,7 +2,9 @@ import io
 import logging
 import os
 import warnings
-from hashlib import md5
+from distutils.version import LooseVersion
+from errno import ESPIPE
+from hashlib import sha256
 from glob import has_magic
 
 from .dircache import DirCache
@@ -33,15 +35,13 @@ class _Cached(type):
     be made for a filesystem instance to be garbage collected.
     """
 
-    cachable = True
-    _extra_tokenize_attributes = ()
-
     def __init__(cls, *args, **kwargs):
         super().__init__(*args, **kwargs)
         # Note: we intentionally create a reference here, to avoid garbage
         # collecting instances when all other references are gone. To really
         # delete a FileSystem, the cache must be cleared.
         cls._cache = {}
+        cls._pid = os.getpid()
 
     def __call__(cls, *args, **kwargs):
         extra_tokens = tuple(
@@ -49,6 +49,9 @@ class _Cached(type):
         )
         token = tokenize(cls, *args, *extra_tokens, **kwargs)
         skip = kwargs.pop("skip_instance_cache", False)
+        if os.getpid() != cls._pid:
+            cls._cache.clear()
+            cls._pid = os.getpid()
         if not skip and cls.cachable and token in cls._cache:
             return cls._cache[token]
         else:
@@ -69,10 +72,14 @@ class _Cached(type):
 
 try:  # optionally derive from pyarrow's FileSystem, if available
     import pyarrow as pa
-
-    up = pa.filesystem.DaskFileSystem
 except ImportError:
     up = object
+else:
+    # only derive from the legacy pyarrow's FileSystem for older pyarrow versions
+    if LooseVersion(pa.__version__) < LooseVersion("2.0"):
+        up = pa.filesystem.DaskFileSystem
+    else:
+        up = object
 
 
 class AbstractFileSystem(up, metaclass=_Cached):
@@ -156,14 +163,16 @@ class AbstractFileSystem(up, metaclass=_Cached):
 
         May require FS-specific handling, e.g., for relative paths or links.
         """
+        if isinstance(path, list):
+            return [cls._strip_protocol(p) for p in path]
         path = stringify_path(path)
         protos = (cls.protocol,) if isinstance(cls.protocol, str) else cls.protocol
         for protocol in protos:
-            path = path.rstrip("/")
             if path.startswith(protocol + "://"):
                 path = path[len(protocol) + 3 :]
-            elif path.startswith(protocol + ":"):
-                path = path[len(protocol) + 1 :]
+            elif path.startswith(protocol + "::"):
+                path = path[len(protocol) + 2 :]
+        path = path.rstrip("/")
         # use of root_marker to make minimum required path, e.g., "/"
         return path or cls.root_marker
 
@@ -273,6 +282,7 @@ class AbstractFileSystem(up, metaclass=_Cached):
         The specific keys, or perhaps a FileInfo class, or similar, is TBD,
         but must be consistent across implementations.
         Must include:
+
         - full path to the entry (without protocol)
         - size of the entry, in bytes. If the value cannot be determined, will
           be ``None``.
@@ -447,7 +457,11 @@ class AbstractFileSystem(up, metaclass=_Cached):
         the same as ``ls(path)``, returning only files.
 
         We support ``"**"``,
-        ``"?"`` and ``"[..]"``.
+        ``"?"`` and ``"[..]"``. We do not support ^ for pattern negation.
+
+        Search path names that contain embedded characters special to this
+        implementation of glob may not produce expected results;
+        e.g., 'foo/bar/*starredfilename*'.
 
         kwargs are passed to ``ls``.
         """
@@ -487,6 +501,10 @@ class AbstractFileSystem(up, metaclass=_Cached):
             depth = 20 if "**" in path else 1
 
         allpaths = self.find(root, maxdepth=depth, withdirs=True, detail=True, **kwargs)
+        # Escape characters special to python regex, leaving our supported
+        # special characters in place.
+        # See https://www.gnu.org/software/bash/manual/html_node/Pattern-Matching.html
+        # for shell globbing details.
         pattern = (
             "^"
             + (
@@ -497,6 +515,10 @@ class AbstractFileSystem(up, metaclass=_Cached):
                 .replace("(", r"\(")
                 .replace(")", r"\)")
                 .replace("|", r"\|")
+                .replace("^", r"\^")
+                .replace("$", r"\$")
+                .replace("{", r"\{")
+                .replace("}", r"\}")
                 .rstrip("/")
                 .replace("?", ".")
             )
@@ -612,22 +634,42 @@ class AbstractFileSystem(up, metaclass=_Cached):
             ``path`` is a dict
         """
         if isinstance(path, str):
-            self.pipe_file(path, value, **kwargs)
+            self.pipe_file(self._strip_protocol(path), value, **kwargs)
         elif isinstance(path, dict):
             for k, v in path.items():
-                self.pipe_file(k, v, **kwargs)
+                self.pipe_file(self._strip_protocol(k), v, **kwargs)
         else:
             raise ValueError("path must be str or dict")
 
-    def cat(self, path, recursive=False, **kwargs):
+    def cat(self, path, recursive=False, on_error="raise", **kwargs):
         """Fetch (potentially multiple) paths' contents
 
         Returns a dict of {path: contents} if there are multiple paths
         or the path has been otherwise expanded
+
+        on_error : "raise", "omit", "return"
+            If raise, an underlying exception will be raised (converted to KeyError
+            if the type is in self.missing_exceptions); if omit, keys with exception
+            will simply not be included in the output; if "return", all keys are
+            included in the output, but the value will be bytes or an exception
+            instance.
         """
         paths = self.expand_path(path, recursive=recursive)
-        if len(paths) > 1 or isinstance(path, list) or paths[0] != path:
-            return {path: self.cat_file(path, **kwargs) for path in paths}
+        if (
+            len(paths) > 1
+            or isinstance(path, list)
+            or paths[0] != self._strip_protocol(path)
+        ):
+            out = {}
+            for path in paths:
+                try:
+                    out[path] = self.cat_file(path, **kwargs)
+                except Exception as e:
+                    if on_error == "raise":
+                        raise
+                    if on_error == "return":
+                        out[path] = e
+            return out
         else:
             return self.cat_file(paths[0])
 
@@ -655,7 +697,6 @@ class AbstractFileSystem(up, metaclass=_Cached):
         """
         from .implementations.local import make_path_posix
 
-        rpath = self._strip_protocol(rpath)
         if isinstance(lpath, str):
             lpath = make_path_posix(lpath)
         rpaths = self.expand_path(rpath, recursive=recursive)
@@ -687,7 +728,11 @@ class AbstractFileSystem(up, metaclass=_Cached):
         """
         from .implementations.local import make_path_posix, LocalFileSystem
 
-        rpath = self._strip_protocol(rpath)
+        rpath = (
+            self._strip_protocol(rpath)
+            if isinstance(rpath, str)
+            else [self._strip_protocol(p) for p in rpath]
+        )
         if isinstance(lpath, str):
             lpath = make_path_posix(lpath)
         fs = LocalFileSystem()
@@ -724,6 +769,7 @@ class AbstractFileSystem(up, metaclass=_Cached):
             out = self.expand_path([path], recursive, maxdepth)
         else:
             out = set()
+            path = [self._strip_protocol(p) for p in path]
             for p in path:
                 if has_magic(p):
                     bit = set(self.glob(p))
@@ -732,9 +778,11 @@ class AbstractFileSystem(up, metaclass=_Cached):
                         out += self.expand_path(p)
                     continue
                 elif recursive:
-                    out |= set(self.find(p, withdirs=True))
-                # TODO: the following is maybe only necessary if NOT recursive
-                out.add(p)
+                    rec = set(self.find(p, withdirs=True))
+                    out |= rec
+                if p not in out and (recursive is False or self.exists(p)):
+                    # should only check once, for the root
+                    out.add(p)
         if not out:
             raise FileNotFoundError(path)
         return list(sorted(out))
@@ -866,7 +914,7 @@ class AbstractFileSystem(up, metaclass=_Cached):
 
     def ukey(self, path):
         """Hash of file properties, to tell if it has changed"""
-        return md5(str(self.info(path)).encode()).hexdigest()
+        return sha256(str(self.info(path)).encode()).hexdigest()
 
     def read_block(self, fn, offset, length, delimiter=None):
         """ Read a block of bytes from
@@ -1053,6 +1101,37 @@ class AbstractFileSystem(up, metaclass=_Cached):
         """Alias of :ref:`FilesystemSpec.get`."""
         return self.get(rpath, lpath, recursive=recursive, **kwargs)
 
+    def sign(self, path, expiration=100, **kwargs):
+        """Create a signed URL representing the given path
+
+        Some implementations allow temporary URLs to be generated, as a
+        way of delegating credentials.
+
+        Parameters
+        ----------
+        path : str
+             The path on the filesystem
+        expiration : int
+            Number of seconds to enable the URL for (if supported)
+
+        Returns
+        -------
+        URL : str
+            The signed URL
+
+        Raises
+        ------
+        NotImplementedError : if method is not implemented for a fileystem
+        """
+        raise NotImplementedError("Sign is not implemented for this filesystem")
+
+    def _isfilestore(self):
+        # Originally inherited from pyarrow DaskFileSystem. Keeping this
+        # here for backwards compatibility as long as pyarrow uses its
+        # legacy ffspec-compatible filesystems and thus accepts fsspec
+        # filesystems as well
+        return False
+
 
 class AbstractBufferedFile(io.IOBase):
     """Convenient class to derive from to provide buffering
@@ -1191,7 +1270,7 @@ class AbstractBufferedFile(io.IOBase):
         """
         loc = int(loc)
         if not self.mode == "rb":
-            raise OSError("Seek only available in read mode")
+            raise OSError(ESPIPE, "Seek only available in read mode")
         if whence == 0:
             nloc = loc
         elif whence == 1:
@@ -1315,8 +1394,9 @@ class AbstractBufferedFile(io.IOBase):
 
         https://docs.python.org/3/library/io.html#io.RawIOBase.readinto
         """
-        data = self.read(len(b))
-        memoryview(b).cast("B")[: len(data)] = data
+        out = memoryview(b).cast("B")
+        data = self.read(out.nbytes)
+        out[: len(data)] = data
         return len(data)
 
     def readuntil(self, char=b"\n", blocks=None):
